package com.almworks.jira.structure.api.attribute.loader.builder;

import com.almworks.jira.structure.api.attribute.AttributeSpec;
import com.almworks.jira.structure.api.attribute.loader.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Set;

public abstract class BaseAttributeLoader<T> implements AttributeLoader<T> {
  @NotNull
  private final AttributeSpec<T> myAttributeSpec;

  @Nullable
  private final Set<AttributeSpec<?>> myDependencies;

  @Nullable
  private final Set<AttributeContextDependency> myContextDependencies;

  @Nullable
  private final AttributeCachingStrategy myCachingStrategy;

  @Nullable
  private final TrailItemSet myGlobalTrail;

  /**
   * Creates the loader. All collections passed must be immutable.
   */
  protected BaseAttributeLoader(@NotNull AttributeSpec<T> attributeSpec, @Nullable Set<AttributeSpec<?>> dependencies,
    @Nullable Set<AttributeContextDependency> contextDependencies, @Nullable AttributeCachingStrategy cachingStrategy,
    @Nullable TrailItemSet globalTrail)
  {
    myAttributeSpec = attributeSpec;
    myDependencies = dependencies;
    myContextDependencies = contextDependencies;
    myCachingStrategy = cachingStrategy;
    myGlobalTrail = globalTrail;
  }

  @NotNull
  @Override
  public AttributeSpec<T> getAttributeSpec() {
    return myAttributeSpec;
  }

  @Nullable
  @Override
  public Set<AttributeSpec<?>> getAttributeDependencies() {
    return myDependencies == null ? AttributeLoader.super.getAttributeDependencies() : myDependencies;
  }

  @Nullable
  @Override
  public Set<AttributeContextDependency> getContextDependencies() {
    return myContextDependencies == null ? AttributeLoader.super.getContextDependencies() : myContextDependencies;
  }

  @Nullable
  @Override
  public AttributeCachingStrategy getCachingStrategy() {
    return myCachingStrategy == null ? AttributeLoader.super.getCachingStrategy() : myCachingStrategy;
  }

  @Nullable
  @Override
  public TrailItemSet getGlobalTrail() {
    return myGlobalTrail == null ? AttributeLoader.super.getGlobalTrail() : myGlobalTrail;
  }
}
