package com.almworks.jira.structure.api.generator.util;

import com.almworks.integers.LongList;
import com.almworks.jira.structure.api.attribute.*;
import com.almworks.jira.structure.api.forest.ForestSpec;
import com.almworks.jira.structure.api.generator.StructureGenerator;
import com.atlassian.jira.util.lang.Pair;
import org.jetbrains.annotations.NotNull;

import java.util.Collection;
import java.util.Collections;

public class GenerationUtil {
  // todo make a method in PregenerationContext instead?
  @NotNull
  public static Pair<RowValues, RecordingAttributeUpdateChecker> getValuesFromSubjectForest(StructureGenerator.GenerationContext context,
    AttributeSpec<?> spec, LongList rows, StructureAttributeService attributeService)
  {
    return getValuesFromSubjectForest(context, Collections.singletonList(spec), rows, attributeService);
  }

  @NotNull
  public static Pair<RowValues, RecordingAttributeUpdateChecker> getValuesFromSubjectForest(
    StructureGenerator.GenerationContext context, Collection<? extends AttributeSpec<?>> specs,
    LongList rows, StructureAttributeService attributeService)
  {
    RowValuesWithUpdateChecker valuesWithChecker;
    ForestSpec transformedSpec = context.getForestSpecBeingTransformed();
    if (transformedSpec != null) {
      // view transformation - can reuse possibly cached values for the forest being transformed
      valuesWithChecker = attributeService.getAttributeValuesWithUpdateChecker(transformedSpec, rows, specs);
    } else {
      // in-structure sorting or grouping, cannot reuse
      valuesWithChecker = attributeService.getAttributeValuesWithUpdateChecker(context.previewForest(), rows, specs, context.getForestSpec());
    }
    RecordingAttributeUpdateChecker checker = new RecordingAttributeUpdateChecker(valuesWithChecker.getUpdateChecker());
    context.addUpdateChecker(checker);
    return Pair.of(valuesWithChecker.getRowValues(), checker);
  }
  
  private GenerationUtil() {}
}
